<?php

declare(strict_types=1);

namespace App\Command;

use App\Entity\Trace;
use App\Model\TraceStatus;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

#[AsCommand(name: 'opengnsys:execute-pending-traces', description: 'Execute pending traces')]
class ExecutePendingTracesCommand extends Command
{
    public function __construct(
        private readonly EntityManagerInterface $entityManager
    ) {
        parent::__construct();
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $startTime = microtime(true);

        $traces = $this->entityManager->getRepository(Trace::class)
            ->findBy(['status' => TraceStatus::PENDING]);

        $count = count($traces);
        $io->info("Found $count pending traces");

        foreach ($traces as $trace) {
            $trace->setStatus(TraceStatus::IN_PROGRESS);
            $this->entityManager->persist($trace);
        }

        $this->entityManager->flush();

        $executionTime = microtime(true) - $startTime;
        $io->success("Updated $count traces to IN_PROGRESS status");
        $io->note("Execution time: " . round($executionTime, 3) . "s");

        return Command::SUCCESS;
    }
} 