<?php

namespace App\Controller\OgAgent;

use App\Dto\Input\CommandExecuteInput;
use App\Dto\Input\MultipleClientsInput;
use App\Entity\Client;
use App\Model\ClientStatus;
use App\Model\CommandTypes;
use App\Model\TraceStatus;
use App\Service\Trace\CreateService;
use Doctrine\ORM\EntityManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class RunScriptAction extends AbstractOgAgentController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(CommandExecuteInput $input): JsonResponse
    {
        /** @var Client $clientEntity */
        foreach ($input->clients as $clientEntity) {
            /** @var Client $client */
            $client = $clientEntity->getEntity();

            if (!$client->getIp()) {
                throw new BadRequestHttpException('IP is required');
            }

            $data = [
                'nfn' => 'EjecutarScript',
                'scp' => base64_encode($input->script),
                'ids' => '0'
            ];

            $response = $this->createRequest(
                method: 'POST',
                url: 'https://'.$client->getIp().':8000/opengnsys/EjecutarScript',
                params: [
                    'json' => $data,
                ],
                token: $client->getToken(),
            );

            if (isset($response['error']) && $response['code'] === Response::HTTP_INTERNAL_SERVER_ERROR) {
                $this->logger->error('Error running script', ['client' => $client->getId(), 'error' => $response['error']]);

                if ($input->queue) {    
                    $inputData = [
                        'script' => $input->script,
                    ];
                    $this->createService->__invoke($client, CommandTypes::RUN_SCRIPT, TraceStatus::PENDING, null, $inputData);
                    continue;
                }
                
                continue;
            }

            $this->logger->info('Powering off client', ['client' => $client->getId()]);

            $jobId = $response['job_id'];

            $this->entityManager->persist($client);
            $this->entityManager->flush();

            $inputData = [
                'script' => $input->script,
            ];

            $this->createService->__invoke($client, CommandTypes::RUN_SCRIPT, TraceStatus::IN_PROGRESS, $jobId, $inputData);
        }

        return new JsonResponse(data: [], status: Response::HTTP_OK);
    }
}