<?php

namespace App\Controller\OgBoot\OgLive;

use App\Controller\OgBoot\AbstractOgBootController;
use App\Entity\OgLive;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class SetDefaultAction extends AbstractOgBootController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(OgLive $data): JsonResponse
    {
        if (!$data->getChecksum()) {
            throw new BadRequestHttpException('Checksum URL is required');
        }

        $params = [
            'json' => [
                'checksum' => $data->getChecksum()
            ]
        ];

        $content = $this->createRequest('PUT', '/ogboot/v1/oglives/default', $params);

        if (isset($content['error']) && $content['code'] === Response::HTTP_INTERNAL_SERVER_ERROR ) {
            throw new BadRequestHttpException('An error occurred: ' . $content['error'] . ' (' . $content['details'] . ')');
        }

        $oldDefaultOgLive = $this->entityManager->getRepository(OgLive::class)->findBy(['isDefault' => true]);

        foreach ($oldDefaultOgLive as $oldOgLive) {
            $oldOgLive->setIsDefault(false);
            $this->entityManager->persist($oldOgLive);
        }

        $data->setIsDefault(true);
        $this->entityManager->persist($data);
        $this->entityManager->flush();

        return new JsonResponse(status: Response::HTTP_OK);
    }
}