<?php

declare(strict_types=1);

namespace App\Controller\OgRepository;

use App\Controller\OgRepository\Image\CreateAuxFilesAction;
use App\Controller\OgRepository\Image\GetAction;
use App\Service\Trace\CreateService;
use Doctrine\ORM\EntityManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
abstract class AbstractOgRepositoryController extends AbstractController
{
    public function __construct(
        protected readonly  EntityManagerInterface   $entityManager,
        protected readonly  HttpClientInterface      $httpClient,
        protected readonly  CreateService            $createService,
        protected readonly  LoggerInterface          $logger,
    )
    {
    }

    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function createRequest(string $method, string $url, array $params = []): array
    {
        $params = array_merge($params, [
            'headers' => [
                'accept' => 'application/json',
                'Content-Type' => 'application/json'
            ],
            'timeout' => 30,
        ]);

        try {
            $response = $this->httpClient->request($method, $url, $params);
            return json_decode($response->getContent(), true);
        } catch (ClientExceptionInterface | ServerExceptionInterface $e) {
            $this->logger->error(sprintf('Client/Server error in request to %s: %s', $url, $e->getMessage()));

            return [
                'code' => Response::HTTP_INTERNAL_SERVER_ERROR,
                'error' => $e->getMessage(),
                'details' => $e->getMessage(),
            ];
        } catch (TransportExceptionInterface $e) {
            $this->logger->error(sprintf('Transport error in request to %s: %s', $url, $e->getMessage()));

            return [
                'code' => Response::HTTP_INTERNAL_SERVER_ERROR,
                'error' => $e->getMessage(),
                'details' => $e->getMessage(),
            ];
        }
    }

}
