<?php

namespace App\Controller\OgRepository\Image;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Dto\Input\BackupImageInput;
use App\Dto\Input\DeleteImageInput;
use App\Entity\Image;
use App\Entity\ImageImageRepository;
use App\Entity\ImageRepository;
use App\Model\CommandTypes;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

#[AsController]
class BackupImageAction extends AbstractOgRepositoryController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(BackupImageInput $input, ImageImageRepository $imageImageRepository): JsonResponse
    {
        $image = $imageImageRepository->getImage();
        $repository = $imageImageRepository->getRepository();

        if (!$image->getName()) {
            throw new BadRequestHttpException('Name is required');
        }

        $params = [
            'json' => [
                'ID_img' => $imageImageRepository->getImageFullsum(),
                'repo_ip' => $input->repoIp,
                'ssh_port' => $repository->getSshPort() ?? '22',
                'remote_path' => $input->remotePath,
                'user' => $repository->getUser() ?? 'opengnsys',
            ]
        ];

        $this->logger->info('Create backup image', ['image' => $image->getName()]);

        $repository = $imageImageRepository->getRepository();

        $content = $this->createRequest('PUT', 'http://'.$repository->getIp().':8006/ogrepository/v1/repo/images', $params);

        if (isset($content['error']) && $content['code'] === Response::HTTP_INTERNAL_SERVER_ERROR ) {
            throw new BadRequestHttpException('Error backing up image: ' . $content['error'] . ' - ' . $content['details']);
        }

        $inputData = [
            'imageName' => $image->getName(),
            'repositoryUuid' => $repository->getUuid(),
            'imageImageRepositoryUuid' => $imageImageRepository->getUuid(),
            'ID_img' => $imageImageRepository->getImageFullsum(),
            'repo_ip' => $input->repoIp,
            'remote_path' => $input->remotePath
        ];

        $this->createService->__invoke($image->getClient(), CommandTypes::BACKUP_IMAGE, TraceStatus::IN_PROGRESS, $content['job_id'], $inputData);

        $imageImageRepository->setStatus(ImageStatus::BACKUP);
        $this->entityManager->persist($imageImageRepository);
        $this->entityManager->flush();


        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }
}