<?php

namespace App\Controller\OgRepository\Image;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Entity\Image;
use App\Entity\ImageImageRepository;
use App\Entity\ImageRepository;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class DeletePermanentAction extends AbstractOgRepositoryController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(ImageImageRepository $data, HttpClientInterface $httpClient): JsonResponse
    {
        if (!$data->getImageFullsum()) {
            throw new BadRequestHttpException('Fullsum is required');
        }

        /** @var ImageRepository $image */
        $image = $data->getImage();

        $this->logger->info('Deleting image', ['image' => $image->getName()]);

        $repository = $data->getRepository();

        $content = $this->createRequest( 'DELETE', 'http://'.$repository->getIp().':8006/ogrepository/v1/images/'.$data->getImageFullsum().'?method=permanent');

        $this->logger->info('Image deleted', ['image' => $image->getName()]);

        $this->entityManager->remove($data);
        $this->entityManager->flush();

        $imageImageCollection = $image->getImageImageRepositories();

        if ($imageImageCollection->isEmpty()) {
            $this->entityManager->remove($image);
            $this->entityManager->flush();
        }

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }
}