<?php

declare(strict_types=1);

namespace App\Controller\OgRepository;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Dto\Input\WoLInput;
use App\Entity\Client;
use App\Entity\Command;
use App\Entity\Image;
use App\Entity\ImageRepository;
use App\Entity\Trace;
use App\Model\ClientStatus;
use App\Model\CommandTypes;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use App\Service\Trace\CreateService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class WoLAction extends AbstractOgRepositoryController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(WoLInput $input): JsonResponse
    {
        foreach ($input->clients as $client) {
            /** @var Client $client */
            $client = $client->getEntity();
            $repository = $client->getRepository();

            if (!$repository->getIp()) {
                throw new BadRequestHttpException('IP is required');
            }

            $params = [
                'json' => [
                    'broadcast_ip' => '255.255.255.255',
                    'mac' => $client->getMac()
                ]
            ];

            $this->logger->info('Sending WoL to client', ['mac' => $client->getMac()]);

            $content = $this->createRequest('POST', 'http://'.$repository->getIp(). ':8006/ogrepository/v1/wol', $params);

            if (isset($content['error']) && $content['code'] === Response::HTTP_INTERNAL_SERVER_ERROR ) {
                $this->logger->error('Error sending WoL to client', ['mac' => $client->getMac()]);
                continue;
            }

            $client->setStatus(ClientStatus::INITIALIZING);
            $this->entityManager->persist($client);
            $this->entityManager->flush();

            $this->createService->__invoke($client, CommandTypes::POWER_ON, TraceStatus::SUCCESS, '', []);
        }

        return new JsonResponse(data: [], status: Response::HTTP_OK);
    }
}
