<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\CommandGroupInput;
use App\Dto\Output\CommandGroupOutput;
use App\Repository\CommandGroupRepository;
use App\Repository\CommandRepository;

readonly class CommandGroupProcessor implements ProcessorInterface
{
    public function __construct(
        private CommandGroupRepository              $commandGroupRepository,
        private ValidatorInterface                  $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): CommandGroupOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): CommandGroupOutput
    {
        if (!($data instanceof CommandGroupInput)) {
            throw new \Exception(sprintf('data is not instance of %s', CommandGroupInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->commandGroupRepository->findOneByUuid($uriVariables['uuid']);
        }

        $command = $data->createOrUpdateEntity($entity);
        $this->validator->validate($command);
        $this->commandGroupRepository->save($command);

        return new CommandGroupOutput($command);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->commandGroupRepository->findOneByUuid($uriVariables['uuid']);
        $this->commandGroupRepository->delete($user);

        return null;
    }
}
