<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\CommandInput;
use App\Dto\Output\CommandOutput;
use App\Repository\CommandRepository;

readonly class CommandProcessor implements ProcessorInterface
{
    public function __construct(
        private CommandRepository           $commandRepository,
        private ValidatorInterface          $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): CommandOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): CommandOutput
    {
        if (!($data instanceof CommandInput)) {
            throw new \Exception(sprintf('data is not instance of %s', CommandInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->commandRepository->findOneByUuid($uriVariables['uuid']);
        }

        $command = $data->createOrUpdateEntity($entity);
        $this->validator->validate($command);
        $this->commandRepository->save($command);

        return new CommandOutput($command);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->commandRepository->findOneByUuid($uriVariables['uuid']);
        $this->commandRepository->delete($user);

        return null;
    }
}
