<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\CommandTaskScriptInput;
use App\Dto\Output\CommandTaskScriptOutput;
use App\Repository\CommandTaskScriptRepository;
use App\Service\CreateTraceService;

readonly class CommandTaskScriptProcessor implements ProcessorInterface
{

    public function __construct(
        private CommandTaskScriptRepository               $commandTaskScriptRepository,
        private ValidatorInterface                        $validator,
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): CommandTaskScriptOutput |null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): CommandTaskScriptOutput
    {
        if (!($data instanceof CommandTaskScriptInput)) {
            throw new \Exception(sprintf('data is not instance of %s', CommandTaskScriptInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->commandTaskScriptRepository->findOneByUuid($uriVariables['uuid']);
        }

        $task = $data->createOrUpdateEntity($entity);
        $this->validator->validate($task);
        $this->commandTaskScriptRepository->save($task);

        return new CommandTaskScriptOutput($task);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->commandTaskScriptRepository->findOneByUuid($uriVariables['uuid']);
        $this->commandTaskScriptRepository->delete($user);

        return null;
    }
}