<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\ClientInput;
use App\Dto\Input\HardwareInput;
use App\Dto\Output\HardwareOutput;
use App\Repository\HardwareRepository;

class HardwareProcessor implements ProcessorInterface
{
    public function __construct(
        private readonly HardwareRepository         $hardwareRepository,
        private readonly ValidatorInterface         $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): HardwareOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): HardwareOutput
    {
        if (!($data instanceof HardwareInput)) {
            throw new \Exception(sprintf('data is not instance of %s', HardwareInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->hardwareRepository->findOneByUuid($uriVariables['uuid']);
        }

        $hardware = $data->createOrUpdateEntity($entity);
        $this->validator->validate($hardware);
        $this->hardwareRepository->save($hardware);

        return new HardwareOutput($hardware);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->hardwareRepository->findOneByUuid($uriVariables['uuid']);
        $this->hardwareRepository->delete($user);

        return null;
    }
}
