<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\HardwareInput;
use App\Dto\Input\HardwareProfileInput;
use App\Dto\Output\HardwareProfileOutput;
use App\Repository\HardwareProfileRepository;

readonly class HardwareProfileProcessor implements ProcessorInterface
{
    public function __construct(
        private HardwareProfileRepository   $hardwareProfileRepository,
        private ValidatorInterface          $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): HardwareProfileOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): HardwareProfileOutput
    {
        if (!($data instanceof HardwareProfileInput)) {
            throw new \Exception(sprintf('data is not instance of %s', HardwareProfileInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->hardwareProfileRepository->findOneByUuid($uriVariables['uuid']);
        }

        $hardwareProfile = $data->createOrUpdateEntity($entity);
        $this->validator->validate($hardwareProfile);
        $this->hardwareProfileRepository->save($hardwareProfile);

        return new HardwareProfileOutput($hardwareProfile);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->hardwareProfileRepository->findOneByUuid($uriVariables['uuid']);
        $this->hardwareProfileRepository->delete($user);

        return null;
    }
}
