<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\HardwareTypeInput;
use App\Dto\Input\MenuInput;
use App\Dto\Input\OperativeSystemInput;
use App\Dto\Input\UserGroupInput;
use App\Dto\Output\HardwareTypeOutput;
use App\Dto\Output\MenuOutput;
use App\Dto\Output\OperativeSystemOutput;
use App\Dto\Output\UserGroupOutput;
use App\Repository\HardwareTypeRepository;
use App\Repository\MenuRepository;
use App\Repository\OperativeSystemRepository;
use App\Repository\UserGroupRepository;

readonly class HardwareTypeProcessor implements ProcessorInterface
{
    public function __construct(
        private HardwareTypeRepository          $hardwareTypeRepository,
        private ValidatorInterface              $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): HardwareTypeOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): HardwareTypeOutput
    {
        if (!($data instanceof HardwareTypeInput)) {
            throw new \Exception(sprintf('data is not instance of %s', MenuInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->hardwareTypeRepository->findOneByUuid($uriVariables['uuid']);
        }

        $hardwareType = $data->createOrUpdateEntity($entity);
        $this->validator->validate($hardwareType);
        $this->hardwareTypeRepository->save($hardwareType);

        return new HardwareTypeOutput($hardwareType);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->hardwareTypeRepository->findOneByUuid($uriVariables['uuid']);
        $this->hardwareTypeRepository->delete($user);

        return null;
    }
}
