<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\ImageRepositoryInput;
use App\Dto\Output\ImageRepositoryOutput;
use App\Repository\ImageRepositoryRepository;

readonly class ImageRepositoryProcessor implements ProcessorInterface
{
    public function __construct(
        private ImageRepositoryRepository     $imageRepository,
        private ValidatorInterface  $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): ImageRepositoryOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): ImageRepositoryOutput
    {
        if (!($data instanceof ImageRepositoryInput)) {
            throw new \Exception(sprintf('data is not instance of %s', ImageRepositoryInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->imageRepository->findOneByUuid($uriVariables['uuid']);
        }

        $repository = $data->createOrUpdateEntity($entity);
        $this->validator->validate($repository);
        $this->imageRepository->save($repository);

        return new ImageRepositoryOutput($repository);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->imageRepository->findOneByUuid($uriVariables['uuid']);
        $this->imageRepository->delete($user);

        return null;
    }
}
