<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\MenuInput;
use App\Dto\Input\UserGroupInput;
use App\Dto\Output\MenuOutput;
use App\Dto\Output\UserGroupOutput;
use App\Repository\MenuRepository;
use App\Repository\UserGroupRepository;

readonly class MenuProcessor implements ProcessorInterface
{
    public function __construct(
        private MenuRepository     $menuRepository,
        private ValidatorInterface $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): MenuOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): MenuOutput
    {
        if (!($data instanceof MenuInput)) {
            throw new \Exception(sprintf('data is not instance of %s', MenuInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->menuRepository->findOneByUuid($uriVariables['uuid']);
        }

        $menu = $data->createOrUpdateEntity($entity);
        $this->validator->validate($menu);
        $this->menuRepository->save($menu);

        return new MenuOutput($menu);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->menuRepository->findOneByUuid($uriVariables['uuid']);
        $this->menuRepository->delete($user);

        return null;
    }
}
