<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\RemoteCalendarInput;
use App\Dto\Output\RemoteCalendarOutput;
use App\Repository\RemoteCalendarRepository;

readonly class RemoteCalendarProcessor implements ProcessorInterface
{
    public function __construct(
        private RemoteCalendarRepository $remoteCalendarRepository,
        private ValidatorInterface  $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): RemoteCalendarOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): RemoteCalendarOutput
    {
        if (!($data instanceof RemoteCalendarInput)) {
            throw new \Exception(sprintf('data is not instance of %s', RemoteCalendarInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->remoteCalendarRepository->findOneByUuid($uriVariables['uuid']);
        }

        $partition = $data->createOrUpdateEntity($entity);
        $this->validator->validate($partition);
        $this->remoteCalendarRepository->save($partition);

        return new RemoteCalendarOutput($partition);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->remoteCalendarRepository->findOneByUuid($uriVariables['uuid']);
        $this->remoteCalendarRepository->delete($user);

        return null;
    }
}
