<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\RemoteCalendarRuleInput;
use App\Dto\Output\RemoteCalendarRuleOutput;
use App\Repository\RemoteCalendarRuleRepository;

readonly class RemoteCalendarRuleProcessor implements ProcessorInterface
{
    public function __construct(
        private RemoteCalendarRuleRepository    $remoteCalendarRuleRepository,
        private ValidatorInterface              $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): RemoteCalendarRuleOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): RemoteCalendarRuleOutput
    {
        if (!($data instanceof RemoteCalendarRuleInput)) {
            throw new \Exception(sprintf('data is not instance of %s', RemoteCalendarRuleInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->remoteCalendarRuleRepository->findOneByUuid($uriVariables['uuid']);
        }

        $partition = $data->createOrUpdateEntity($entity);
        $this->validator->validate($partition);
        $this->remoteCalendarRuleRepository->save($partition);

        return new RemoteCalendarRuleOutput($partition);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->remoteCalendarRuleRepository->findOneByUuid($uriVariables['uuid']);
        $this->remoteCalendarRuleRepository->delete($user);

        return null;
    }
}
