<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\SoftwareProfileInput;
use App\Dto\Output\SoftwareProfileOutput;
use App\Repository\SoftwareProfileRepository;

readonly class SoftwareProfileProcessor implements ProcessorInterface
{
    public function __construct(
        private SoftwareProfileRepository   $softwareProfileRepository,
        private ValidatorInterface          $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): SoftwareProfileOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): SoftwareProfileOutput
    {
        if (!($data instanceof SoftwareProfileInput)) {
            throw new \Exception(sprintf('data is not instance of %s', SoftwareProfileInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->softwareProfileRepository->findOneByUuid($uriVariables['uuid']);
        }

        $softwareProfile = $data->createOrUpdateEntity($entity);
        $this->validator->validate($softwareProfile);
        $this->softwareProfileRepository->save($softwareProfile);

        return new SoftwareProfileOutput($softwareProfile);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->softwareProfileRepository->findOneByUuid($uriVariables['uuid']);
        $this->softwareProfileRepository->delete($user);

        return null;
    }
}
