<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\UserGroupInput;
use App\Dto\Output\UserGroupOutput;
use App\Repository\UserGroupRepository;

class UserGroupProcessor implements ProcessorInterface
{
    public function __construct(
        private readonly UserGroupRepository    $userGroupRepository,
        private readonly ValidatorInterface     $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): UserGroupOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): UserGroupOutput
    {
        if (!($data instanceof UserGroupInput)) {
            throw new \Exception(sprintf('data is not instance of %s', UserGroupInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->userGroupRepository->findOneByUuid($uriVariables['uuid']);
        }

        $userGroup = $data->createOrUpdateEntity($entity);
        $this->validator->validate($userGroup);
        $this->userGroupRepository->save($userGroup);

        return new UserGroupOutput($userGroup);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->userGroupRepository->findOneByUuid($uriVariables['uuid']);
        $this->userGroupRepository->delete($user);

        return null;
    }
}
